#include <stdio.h>

/** SEGMENT DESCRIPTOR **/
/*  0 ~ 15 : Segment size limit 0-15
 * 16 ~ 31 : Segment base address bits 0-15
 * 32 ~ 39 : Segment base address bits  16-23
 * 40 ~ 43 : type
 * 44      : 0 = system segment 1 = code or data
 * 45 ~ 46 : privilege level
 * 47      : present
 * 48 ~ 51 : Segment size limit 16-19
 * 52      : system bit available for use by the OS
 * 53      : 0
 * 54      : default size
 * 55      : granularity
 * 56 ~ 63 : segment base address 24-31
 */
typedef struct stDescriptor_t {
		unsigned short wLimig_15_0;
		unsigned short wBase_15_0;
		unsigned char wBase_23_16;
		unsigned char bArrtibuteLow;
		unsigned char bAttributeHigh;
		unsigned char bBase_31_24;
} stDescriptor;


/** Page Directory Entry **/

/*  0      : 1 = page table pointed to is present physical memory
 *  1      : 1 = read/write 0 = read only
 *  2      : 0 = supervisor 1 = user
 *  3      : write through (used when caching enable)
 *  4      : 1 = cache disabled
 *  5      : 1 = page table has recently been accessed
 *  6      : global flag
 *  7      : 0
 *  8      : 0 = page size is 4KB
 *  9 ~ 11 : available for OS
 * 12 ~ 31 : Page table base address
 */
typedef struct stPageGlobalDIR_t {
		unsigned long dwPGD;
} stPageGlobalDIR;

typedef struct stPageMiddleDIR_t {
		unsigned long dwPMD;
} stPageMiddleDIR;


/** Page Table Entry **/

/*  0      : 1 = 4KB page is present in physical memory
 *  1      : 1 = read/write 0 = read only
 *  2      : 0 = supervisor 1 = user
 *  3      : write through (used when caching enable)
 *  4      : 1 = cache disabled
 *  5      : 1 = page table has recently been accessed
 *  6      : page has been written to when set to 1 (DIRTY)
 *  7      : 0
 *  8      : global flag 
 *  9 ~ 11 : available for OS
 * 12 ~ 31 : Page base address
 */
typedef struct stPageTableEntry_t {
		unsigned long dwPTE;
} stPageTableEntry;
		


stDescriptor GDT[8192] = {
		{0, 0, 0, 0, 0, 0},
		{0xffff, 0x0, 0x0, 0x9a, 0xcf, 0x0},
		{0xffff, 0x0, 0x0, 0x92, 0xcf, 0x0},
		{0xffff, 0x0, 0x0, 0xfa, 0xcf, 0x0},
		{0xffff, 0x0, 0x0, 0xf2, 0xcf, 0x0},
};


stPageGlobalDIR PDG[1024];
stPageTableEntry PTE[1024];


void InitGDT(void)
{
		
#ifdef RELEASE
		/* set GDTR */

		/* The GDTR is 48 bits in size.
		 * 0 ~ 15 (16 bits) : contain the size limit of the GDT.
		 * 16 ~ 47 (32 bits) : GDT base address 
		 */

		asm {

		}

#endif

		
		
}



int main(void)
{

		InitGDT();

		return 0;

}


